@@ -18,6 +18,93 @@
 
 
 
+#' Using the Concorde TSP Solver
+#'
+#' The Concorde TSP Solver package contains several solvers.  Currently,
+#' interfaces to the Concorde solver (Applegate et al. 2001), one of the most
+#' advanced and fastest TSP solvers using branch-and-cut, and the Chained
+#' Lin-Kernighan (Applegate et al. 2003) implementation are provided in
+#' \pkg{TSP}. Concorde can solve [TSP]s and [ETSP]s directly. [ATSP]s are
+#' reformulated as larger TSP's and then solved.
+#'
+#'
+#' **Installation of Concorde**
+#'
+#' The Concorde TSP Solver is freely available for academic research.
+#' It is not included in the \pkg{TSP} R package and has
+#' to be obtained separately from the
+#' [Concorde download page](http://www.math.uwaterloo.ca/tsp/concorde/downloads/downloads.htm).
+#' Either download the precompiled executables and place them in a suitable
+#' directory (make sure they are executable), or you can get the source code and
+#' compile the program on your own. \pkg{TSP} needs to know where the executables are.
+#' There are two options:
+#' 1. use `concorde_path()` to set the path to the
+#'   directory containing the executables for concorde and linkern, or
+#' 2. make
+#'   sure that the executables are in the search path stored in the `PATH`
+#'   environment variable (see [Sys.setenv()]).
+#'
+#' **Using Concorde for `solve_TSP()`**
+#'
+#' [solve_TSP()] uses [write_TSPLIB()] to write the TSP for
+#' Concorde and tries to find the appropriate `precision` value (digits
+#' after the decimal point) to convert the provided distances into the needed
+#' integer value range. The `precision` value can also be specified in
+#' `control` in [solve_TSP()] with method Concorde. Warning
+#' messages will alert the user if the conversion to integer values results
+#' into rounding errors that are worse then what is specified in the
+#' `precision` control parameter.
+#'
+#' To get a list of all available command line options which can be used via
+#' the `clo` option for `solve_TSP` use `concorde_help()` and
+#' `linkern_help()`.  Several options (\option{-x}, \option{-o},
+#' \option{-N}, \option{-Q}) are not available via [solve_TSP()] since they
+#' are used by the interface.
+#'
+#' If Concorde takes too long, then you can kill the 'concorde' process via your
+#' operating system and you can continue with R.
+#'
+#' @family TSP
+#'
+#' @name Concorde
+#' @aliases Concorde concorde concorde_path concorde_help linkern_help
+#'
+#' @param path a character string with the path to the directory where the
+#' executables are installed.
+#' @returns Nothing.
+#' @author Michael Hahsler
+#' @references Concorde home page,
+#' \url{http://www.math.uwaterloo.ca/tsp/concorde/}
+#'
+#' David Applegate, Robert Bixby, Vasek Chvatal, William Cook (2001): TSP cuts
+#' which do not conform to the template paradigm, Computational Combinatorial
+#' Optimization, M. Junger and D. Naddef (editors), Springer-Verlag.
+#'
+#' David Applegate and William Cook and Andre Rohe (2003): Chained
+#' Lin-Kernighan for Large Traveling Salesman Problems, \emph{INFORMS Journal
+#' on Computing}, \bold{15}, 82--92.
+#' @keywords documentation
+#' @examples
+#'
+#' \dontrun{
+#' ## see if Concorde is correctly installed
+#' concorde_path()
+#'
+#'
+#' ## set path to the Concorde executible if it is not in the search PATH
+#' ## Example:
+#' ## concorde_path("~/concorde/")
+#'
+#' concorde_help()
+#'
+#' data("USCA312")
+#'
+#' ## run concorde in verbose mode (-v) with fast cuts only (-V)
+#' solve_TSP(USCA312, method = "concorde", control = list(clo = "-v -V"))
+#' }
+#'
+NULL
+
 
 ## prepare distances as integers in the appropriate range [0..MAX]
 .prepare_dist_concorde <- function(x, MAX, precision) {
@@ -94,7 +181,7 @@ tsp_concorde <- function(x, control = NULL){
   ## change working directory
 
   ## do the call and read back result
-  ## we do not check return values of concorde since they are not
+  ## we do not check return values of Concorde since they are not
   ## very consistent
   system2(control$exe,
     args =  paste("-x", control$clo, "-o", tmp_file_out, tmp_file_in),
@@ -104,7 +191,7 @@ tsp_concorde <- function(x, control = NULL){
 
 
   if(!file.access(tmp_file_out) == 0)
-    stop("Problems with reading Concorde's output.\nIs concorde properly installed?\nFor details see ? Concorde")
+    stop("Concorde has not produced a result file.\nIs concorde properly installed? (see ? Concorde)\nDid Concorde finish without an error or being interupted?")
   ##else cat("Concorde done.\n")
 
   order <- scan(tmp_file_out, what = integer(0), quiet = TRUE)
@@ -167,7 +254,7 @@ tsp_linkern <- function(x, control = NULL){
   write_TSPLIB(x, file = tmp_file_in, precision = 0)
 
   ## do the call and read back result
-  ## we do not check return values of concorde since they are not
+  ## we do not check return values of Concorde since they are not
   ## very consistent
   system2(control$exe, args =  paste("-o",
     tmp_file_out, control$clo, tmp_file_in),
@@ -175,7 +262,7 @@ tsp_linkern <- function(x, control = NULL){
     stderr = if(control$verbose) "" else FALSE)
 
   if(!file.access(tmp_file_out) == 0)
-    stop("Problems with reading linkern's output. Is linkern properly installed?")
+    stop("Linkern has not produced a result file.\nIs linkern properly installed?\nDid linkern finish without an error or being interrupted?")
   ##else cat("Concorde done.\n")
 
   order <- read.table(tmp_file_out)[,1]
@@ -189,19 +276,10 @@ tsp_linkern <- function(x, control = NULL){
   order
 }
 
-
-## get help page
-concorde_help <- function() {
-  cat("The following options can be specified in solve_TSP with method \"concorde\" using clo in control:\n\n")
-  system2(.find_exe(NULL, "concorde"), args = "")
-}
-
-linkern_help <- function() {
-  cat("The following options can be specified in solve_TSP with method \"linkern\" using clo in control:\n\n")
-  system2(.find_exe(NULL, "linkern"), args = "")
-}
-
 ## path
+## path
+#' @rdname Concorde
+#' @export
 concorde_path <- local({
   .path <- NULL
   function(path){
@@ -235,7 +313,24 @@ concorde_path <- local({
 })
 
 
-## helper to find the concorde executable
+## get help page
+#' @rdname Concorde
+#' @export
+concorde_help <- function() {
+  cat("The following options can be specified in solve_TSP with method \"concorde\" using clo in control:\n\n")
+  system2(.find_exe(NULL, "concorde"), args = "")
+}
+
+#' @rdname Concorde
+#' @export
+linkern_help <- function() {
+  cat("The following options can be specified in solve_TSP with method \"linkern\" using clo in control:\n\n")
+  system2(.find_exe(NULL, "linkern"), args = "")
+}
+
+
+
+## helper to find the 'concorde' executable
 .find_exe <- function(exe = NULL, prog) {
   ## if not specified
   if(is.null(exe)) {
